diff --git a/sys/dev/etherswitch/rtl8366rb.c b/sys/dev/etherswitch/rtl8366rb.c
index f41a425..6f3e378 100644
--- a/sys/dev/etherswitch/rtl8366rb.c
+++ b/sys/dev/etherswitch/rtl8366rb.c
@@ -139,7 +139,7 @@ rtl8366rb_attach(device_t dev)
 
 	sc = device_get_softc(dev);
 	sc->dev = dev;
-	mtx_init(&sc->sc_mtx, device_get_nameunit(dev), MTX_NETWORK_LOCK,
+	mtx_init(&sc->sc_mtx, device_get_nameunit(dev), "rtl8366rb",
 	    MTX_DEF);
 	rtl8366rb_init(dev);
 	smi_read(dev, RTL8366RB_CVCR, &rev);
@@ -201,16 +201,60 @@ rtl8366rb_detach(device_t dev)
 }
 
 static void
+rtl833rb_miipollstat(struct rtl8366rb_softc *sc)
+{
+	int i;
+	struct mii_data *mii;
+	struct mii_softc *child;
+	uint16_t value;
+	int portstatus;
+
+	for (i = 0; i < RTL8366RB_NUM_PORTS-1; i++) {
+		mii = device_get_softc(sc->miibus[i]);
+		if ((i % 2) == 0) {
+			smi_read(sc->dev, RTL8366RB_PLSR_BASE + i/2, &value);
+			portstatus = value & 0xff;
+		} else {
+			portstatus = (value >> 8) & 0xff;
+		}
+		mii->mii_media_status = IFM_AVALID;
+		mii->mii_media_active = IFM_ETHER;
+		if ((portstatus & RTL8366RB_PLSR_LINK) != 0) {
+			switch (portstatus & RTL8366RB_PLSR_SPEED_MASK) {
+			case RTL8366RB_PLSR_SPEED_10:
+				mii->mii_media_active |= IFM_10_T;
+				break;
+			case RTL8366RB_PLSR_SPEED_100:
+				mii->mii_media_active |= IFM_100_TX;
+				break;
+			case RTL8366RB_PLSR_SPEED_1000:
+				mii->mii_media_active |= IFM_1000_T;
+				break;
+			}
+			if ((portstatus & RTL8366RB_PLSR_FULLDUPLEX) != 0)
+				mii->mii_media_active |= IFM_FDX;
+			else
+				mii->mii_media_active |= IFM_HDX;
+			if ((portstatus & RTL8366RB_PLSR_TXPAUSE) != 0)
+				mii->mii_media_active |= IFM_ETH_TXPAUSE;
+			if ((portstatus & RTL8366RB_PLSR_RXPAUSE) != 0)
+				mii->mii_media_active |= IFM_ETH_RXPAUSE;
+		} else
+			mii->mii_media_active |= IFM_NONE;
+		LIST_FOREACH(child, &mii->mii_phys, mii_list) {
+			if (IFM_INST(mii->mii_media.ifm_cur->ifm_media) != child->mii_inst)
+				continue;
+			(void)PHY_SERVICE(child, mii, MII_POLLSTAT);
+		}
+	}	
+}
+
+static void
 rtl8366rb_tick(void *arg)
 {
 	struct rtl8366rb_softc *sc = arg;
-	int i;
 	
-	for (i=0; i < RTL8366RB_NUM_PORTS-1; i++) {
-		if (sc->miibus[i] != NULL)
-			mii_tick(device_get_softc(sc->miibus[i]));
-			mii_pollstat(device_get_softc(sc->miibus[i]));
-	}
+	rtl833rb_miipollstat(sc);
 	callout_reset(&sc->callout_tick, hz, rtl8366rb_tick, sc);
 }
 
@@ -435,10 +479,7 @@ rtl8366rb_ifmedia_upd(struct ifnet *ifp)
 {
 	struct rtl8366rb_softc *sc = ifp->if_softc;
 	struct mii_data *mii = device_get_softc(sc->miibus[ifp->if_dunit]);
-	struct mii_softc *miisc;
 	
-	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		PHY_RESET(miisc);
 	mii_mediachg(mii);
 	return (0);
 }
diff --git a/sys/dev/etherswitch/rtl8366rbvar.h b/sys/dev/etherswitch/rtl8366rbvar.h
index f14373a..df17cb6 100644
--- a/sys/dev/etherswitch/rtl8366rbvar.h
+++ b/sys/dev/etherswitch/rtl8366rbvar.h
@@ -65,7 +65,8 @@
 #define RTL8366RB_PLSR_SPEED_10		0x00
 #define RTL8366RB_PLSR_SPEED_100	0x01
 #define RTL8366RB_PLSR_SPEED_1000	0x02
-#define RTL8366RB_PLSR_FULLDUPLEX	0x10
+#define RTL8366RB_PLSR_FULLDUPLEX	0x08
+#define RTL8366RB_PLSR_LINK		0x10
 #define RTL8366RB_PLSR_TXPAUSE		0x20
 #define RTL8366RB_PLSR_RXPAUSE		0x40
 #define RTL8366RB_PLSR_NO_AUTO		0x80
